/*******************************************************************/
/*                                                      21.03.2009 */
/*                                                                 */
/* Hier werden die Funktionen zum ansprechen des K/L-Busses        */
/* nach ISO-9141 fuer Lancia definiert                             */
/*                                                                 */
/* Momentan wird nur das Auslesen des Klimasteuergeraet            */
/* unterstuztzt. Dieses Steuergeraet benutzt sowohl die K wie auch */
/* die L-Line!                                                     */
/*                                                                 */
/*                                                                 */
/*                                                                 */
/*                                                                 */
/*                                                                 */
/*******************************************************************/
#pragma pack(2)

#include <stdio.h>
#include <string.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>


#include "AirCon.h"
#include "rs232.h"
#include "KL_Line.h"




/*******************************************************************/
/* Setzt die K-Leitung auf HighPegel/12V/Logisch1                  */
/*******************************************************************/
void set_K(void)
{

}

/*******************************************************************/
/* Setzt die K-Leitung auf LowPegel/0V/Logisch0                    */
/*******************************************************************/
void clr_K(void)
{

}

/*******************************************************************/
/* Setzt die L-Leitung auf HighPegel/12V/Logisch1                  */
/*******************************************************************/
void set_L(void)
{

  ResetSerialRTS();  /* Pin4 des PalmConnector */

}

/*******************************************************************/
/* Setzt die L-Leitung auf LowPegel/0V/Logisch0                    */
/*******************************************************************/
void clr_L(void)
{
  /* Der RTS Ausgang des Palmpiloten wird hier auf +5V gesetzt */
  /* das fuehrt dazu das ein Transistor die L-Line auf Masse   */
  /* zieht und so eine 0 ausgibt. Mit anderen Worten die       */
  /* Bedeutung ist hier invertiert weil unsere Hardware ein    */
  /* weiteres mal intertiert!                                  */
  SetSerialRTS(); /* Pin4 des PalmConnector */
}


/*******************************************************************/
/* Allgemeine Initialisierung des KL-Busses                        */
/*******************************************************************/
void kl_init(void)
{

  /* Beide Leitung sind Defaultmaessig auf Logisch1 */
  set_K();
  set_L();

  AirCon.Status = Not_connected;
  AirCon.State  = State_Stop; 

  AirCon.FrameCounter = 0;
  AirCon.PayLoad = PayLoad_Ready;
}

/*******************************************************************/
/* Diese Funktion soll 200ms warten.                               */
/* Koennte evenutell noch kritisch sein weil sie den gesammten     */
/* Prozessor fuer 200ms schlafen legt, der User also keine         */
/* Eingaben machen kann.                                           */
/*******************************************************************/
void kl_wait200ms(void)
{

  SysTaskDelay( (SysTicksPerSecond() * 2)  / 10 );

}

/*******************************************************************/
/* Fuer die Initalisierung des Busses ist es notwendig ein Byte    */
/* mit nur 5Baud auf der L-Line zu senden. Also 1Bit=200ms         */
/* Ich moechte wirklich mal gerne wissen wer sich diesen hirnlosen */
/* scheiss ausgedacht hat!                                         */
/* Dadurch dauert der Verbindungsaufbau mindestens 2s              */
/*******************************************************************/
void kl_send5Baud(unsigned char data)
{ int i;

  /* StartBit */
  clr_L(); kl_wait200ms();

  /* Nun die acht Datenbits */
  for (i=8;i>0;i--)
    {

      if ( (data & 0x01) == 0x00) 
	{
	  clr_L();
	}
      else  
	{
	  set_L();
	};

      kl_wait200ms();

      data =  data  >> 1  ;  /* Ein Bit nach links */
    }

  /* StopBit */
  set_L(); 

  /* Wir duerfen hier nicht warten weil das Steuergeraet */
  /* sehr fix zuruecksendet */
  /* kl_wait200ms(); */

}


/*******************************************************************/
/* Erwartet einen Datenframe vom Steuergeraet                      */     
/*******************************************************************/
int  kl_getframe(struct ISO_Frame *SteuerDats)
{ int i;
  unsigned char Checksum;

  /*  char buffer[40]; */

 SteuerDats->Error = No_Error;

 /* BlockLaenge einlesen */
 if (rs232_receive( &(SteuerDats->Blocklen),1,10) == FALSE) 
   { SteuerDats->Error = BUS_Timeout;
     return(FALSE);
   };
 Checksum=SteuerDats->Blocklen;



 /* Title einlesen */
 if (rs232_receive( &(SteuerDats->Title),1,10) == FALSE) 
   { SteuerDats->Error = BUS_Timeout;
     return(FALSE);
   };
 Checksum+=SteuerDats->Title;

 /* Nicht mehr lesen als wir verdauen koennen */
 if ( (SteuerDats->Blocklen -1 )>Parameter_Len) 
   { SteuerDats->Error = FrameSize;

   /*
	   sprintf(buffer,"Frame: %i:  %i  ",SteuerDats->Blocklen,Parameter_Len);
  	   WinDrawChars(buffer,StrLen(buffer),  40, 120);
   */
     return(FALSE);
   };

 /* Parameter einlesen */
 for (i=0; i<(SteuerDats->Blocklen -2); i++)
   {
     if (rs232_receive( &(SteuerDats->Parameter[i]),1,10) == FALSE) 
       { SteuerDats->Error = No_Data;
         return(FALSE);
       }
     else
       {
	 Checksum+=SteuerDats->Parameter[i];
       }
   };

 /* Pruefsumme einlesen */
 if (rs232_receive( &(SteuerDats->Checksum),1,10) == FALSE) 
   { SteuerDats->Error = No_Checksum;
     return(FALSE);
   }

 if ( Checksum != SteuerDats->Checksum)
   { SteuerDats->Error = Wrong_Checksum;
     return(FALSE);
   }


 return(TRUE);
}

/*******************************************************************/
/* Sendet ein ACK an das Steuergeraet                              */     
/*******************************************************************/
int kl_send_ACK()
{
  /* ACK-Befehl inklusive Laenge und Pruefsumme */
  unsigned char buffer[3] = {0x02, 0x44, 0x46}; 

  /* Befehl abschicken */
  if (rs232_buffersend(buffer, 3) == FALSE) 
    {  
        return(FALSE);
    };

  /* Warten bis alle Bytes gesendet wurden */
  if (rs232_wait(10) == FALSE ) 
    {
      return(FALSE);
    };

  /* Das Steuergeraet schickt die Antwort 28ms nach dem */
  /* Aussenden des letzten Bytes! */

  AirCon.FrameCounter +=1;
 
  /* Diese Zeichen stehen jetzt auch in unserem Empfangsbuffer */
  /* aber wir brauchen sie nicht */
  rs232_buffclr();


  return(TRUE);
}

/*******************************************************************/
/* Sendet ein DatenPaket an das Steuergeraet                       */     
/* Berechnet auch die Pruefsumme!                                  */
/*******************************************************************/
int kl_send_Payload()
{ int i;
  unsigned char buffer[Parameter_Len+4];
  unsigned char psum;

  psum=0;

  /* Laenge des Befehls/Frames  */
  buffer[0] = AirCon.PayLoadFrame.Blocklen; psum +=buffer[0];

  /* Kein Bufferueberlauf erlauben */
  if (buffer[0] >= (Parameter_Len+4)) return(FALSE);

  /* Befehl */
  buffer[1] = AirCon.PayLoadFrame.Title; psum +=buffer[1];

  i=2;

  /* Wenn vorhanden noch Daten/Parameter kopieren */
  while (i<buffer[0])
    {
      buffer[i] = AirCon.PayLoadFrame.Parameter[i-2];
      psum +=buffer[i];
      i++;
    }

  /* Pruefsumme eintragen */
  buffer[i]=psum;

  /* Befehl abschicken */
  if (rs232_buffersend(buffer, i+1) == FALSE) 
    {  
        return(FALSE);
    };

  /* Warten bis alle Bytes gesendet wurden */
  if (rs232_wait(10) == FALSE ) 
    {
      return(FALSE);
    };

  /* Das Steuergeraet schickt die Antwort 28ms nach dem */
  /* Aussenden des letzten Bytes! */

  AirCon.FrameCounter +=1;
 
  /* Diese Zeichen stehen jetzt auch in unserem Empfangsbuffer */
  /* aber wir brauchen sie nicht */
  rs232_buffclr();


  return(TRUE);
}

/*******************************************************************/
/* Baut eine Datenverbindung zum Klimasteuergeraet auf             */     
/*******************************************************************/
int kl_klimaconnect(void)
{ 

  AirCon.Status = Not_connected;

  /* Buffer loeschen */
  rs232_buffclr();

  /* Sendet die Adresse des Steuergeraetes mit 5Baud auf L-Line */
  kl_send5Baud( KL_ADRESS_CLIMA  );


  /* Jetzt sollte das Steuergeraet innerhalb der naechsten */
  /* zwei Sekunden ein paar Daten mit 4800Baud auf der     */
  /* K-Line schicken */

  /* 6Bytes, 2Sekunden */
  if (rs232_receive(AirCon.Preamble,Preamble_Len,20) == FALSE)
    {
      AirCon.Status = No_Preamble;
      return(FALSE);
    };

  /* Zwischen Empfang der Preamble und dem Zuruecksenden des */
  /* invertierten Anwortbytes muss eine Wartezeit von 10ms   */
  /* bis 200ms liegen                                        */
   SysTaskDelay( 5 ); 

  /* Wenn wir bis hier gekommen sind dann hat uns das       */
  /* Steuergeraet immerhin schonmal 6Bytes zurueckgeschickt */
  /* Als naechstes erwartet es von uns das Byte[2] invertiert */
  
   if (rs232_bytesend(  ~(AirCon.Preamble[2])  ) == FALSE )
    {
      AirCon.Status = Timeout;
      return(FALSE);
    };

   /* Warten bis das Zeichen gesendet ist */
   rs232_wait(10);

   /* Buffer erneut loeschen, sonst steht unser selbst      */
   /* gesendetes Byte als erstes drin!                      */
   rs232_buffclr();

   /* Jetzt dauert es etwa 80ms bis der Frame vom Steuergeraet kommt */ 

   /* Einlesen des DatenFrames */
   if ( kl_getframe(&(AirCon.DataFrame)) == FALSE)
     {
       AirCon.Status = Frame_Error;
       return(FALSE);
     };

   /* Seriennummer kopieren */
   StrNCopy(AirCon.SerNum, AirCon.DataFrame.Parameter, 8);
   AirCon.SerNum[8]=0; 

   /* Hardwareversion kopieren */
   AirCon.HW_Version=AirCon.DataFrame.Parameter[8];

   /* Softwareversion kopieren */
   AirCon.SW_Version=AirCon.DataFrame.Parameter[9];

   /* Herstellungswoche */
   StrNCopy(AirCon.Week, AirCon.DataFrame.Parameter+8, 2);
   AirCon.Week[2]=0;

   AirCon.Year = AirCon.DataFrame.Parameter[12];

  /* Wir haben jetzt eine Verbindung aufgebaut           */
  /* Von nun an muss spaetestens alle 200ms eine Abfrage */
  /* an das Steuergeraet geschickt werden, oder das      */
  /* Steuergeraet beendet die Verbindung von sich aus!   */
  AirCon.State  = State_Start; 
  AirCon.Status = Connected;
  AirCon.FrameCounter = 0;
  AirCon.PayLoad = PayLoad_Ready;
  AirCon.LastParam = PARAM_READY;
  AirCon.Active_Diag = FALSE;
  return(TRUE);
}

/*******************************************************************/
/* Liesst die eingetroffene Antwort aus dem Datenpuffer der RS232  */
/*******************************************************************/
int kl_parse_Answer(struct ISO_Frame *SteuerDat)
{
   /* Einlesen des DatenFrames */
   if ( kl_getframe(SteuerDat) == FALSE)
     {       AirCon.Status = Frame_Error;
       return(FALSE);
     };

  return(TRUE);
}

/*******************************************************************/
/* Nachdem eine Verbindung mit dem Steuergeraet aufgebaut wurde,   */
/* muss diese Funktion spaetestens alle 200ms aufgerufen werden!   */
/* Es darf aber auch nicht oefter als alle 10ms angefragt werden.  */
/*                                                                 */
/* Wenn es eine Datenanforderung des Nutzers gibt, so sendet sie   */
/* ans Steuergeraet und legt die Antwort bereit.                   */
/*                                                                 */
/* Gibt es nichts zutun, so muss trotzdem ein Acknowledge          */
/* angefordert werden, oder die Verbindung bricht zusammen!        */     
/*                                                                 */
/* Der Aufruf dieser Funktion erfolgt vom der MainEventschleife    */
/* alle 150ms. Und zwar immer nach Programmstart!                  */
/* Sie muss daher selber wissen ob sie bereits mit dem Steuergeraet*/
/* PingPong spielen muss oder nicht.                               */
/*******************************************************************/
int kl_Frame_exchange(void)
{ char buffer[40];

  /* Wenn wir keine Verbindung haben dann brauchen wir  */
  /* auch nichts zu machen. Wichtiger noch, wir duerfen */
  /* auch nichts machen da vielleicht die RS232 noch    */
  /* garnicht offen ist!                                */
  if (AirCon.Status != Connected) {
    return(TRUE);
  };

  switch(AirCon.State) 
    {
    
      /* Es soll ein neues Paeckchen angefordert werden. */
    case State_Start : 
      if ( AirCon.PayLoad == PayLoad_Start)
	{
	  if ( kl_send_Payload() == FALSE) 
	    { AirCon.Status = Not_connected; /* Fehler bricht Verbindung ab! */    
	    /*   WinDrawChars("SendErrA ",8,  10, 20); */
	    break;  
	    };
	  AirCon.PayLoad = PayLoad_Wait;
	}
      else
	{
	  if ( kl_send_ACK() == FALSE) 
	    { AirCon.Status = Not_connected; /* Fehler bricht Verbindung ab! */    
	    /*   WinDrawChars("SendErrB ",8,  10, 30); */
	    break;  
	    };
	};

      AirCon.State = State_Waiting;
      break;

      /* Es sollte nun eine Antwort da sein */
    case State_Waiting :

      /* Ist das das normale ACK-PingPong, oder etwas die Antwort */
      /* auf angeforderte Daten?                                  */
      if ( AirCon.PayLoad == PayLoad_Wait)
	{
	  if (kl_parse_Answer(&(AirCon.PayLoadFrame)) == FALSE) 
	    { AirCon.Status = Not_connected; /* Fehler bricht Verbindung ab! */    
	    /*    WinDrawChars("SendErrC ",8,  10, 40);  */
	    };
	  AirCon.PayLoad = PayLoad_Ready;
	}
      else
	{
	  if (kl_parse_Answer(&(AirCon.DataFrame)) == FALSE) 
	    { AirCon.Status = Not_connected; /* Fehler bricht Verbindung ab! */    
	    /*   WinDrawChars("SendErrD ",8,  10, 50);  */
	    };
	};

      AirCon.State = State_Start;
      break;

      /* Sollte eigentlich niemals vorkommen, aber man weiss ja nie */
    default:
      AirCon.Status = Not_connected;
      AirCon.State = State_Start;
      AirCon.PayLoad = PayLoad_Ready;
      break;
    }  


  return(TRUE);
}

/*******************************************************************/
/* Liefert TRUE wenn wir frische Daten vom Steuergeraet da haben   */
/*******************************************************************/
int kl_CommREADY(void)
{

  if (AirCon.PayLoad == PayLoad_Ready) return(TRUE);

  return(FALSE);
}

/*******************************************************************/
/* Vordert den Status vom Steuergeraet an                          */
/*******************************************************************/
void kl_Command_ReadStatus(void)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen = 0x02;
  AirCon.PayLoadFrame.Title    = COMMAND_READ_STATUS;

  AirCon.PayLoad = PayLoad_Start;


}

/*******************************************************************/
/* Liesst Fehlermeldungen aus Steuergeraet                         */
/*******************************************************************/
void kl_Command_ReadError(void)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen = 0x02;
  AirCon.PayLoadFrame.Title    = COMMAND_READ_ERROR;

  AirCon.PayLoad = PayLoad_Start;


}

/*******************************************************************/
/* Loescht im Steuergeraet gespeicherte Fehler                     */
/*******************************************************************/
void kl_Command_ClearError(void)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen = 0x02;
  AirCon.PayLoadFrame.Title    = COMMAND_CLEAR_ERROR;

  AirCon.PayLoad = PayLoad_Start;


}

/*******************************************************************/
/* Nur zu Testzwecken                                              */     
/*******************************************************************/
void kl_test(void)
{


  kl_klimaconnect();
}


/*******************************************************************/
/* Liesst Parameter aus Steuergeraet.                              */
/* Erwartet dazu den Parameter der gelesen werden soll             */
/*******************************************************************/
void kl_Command_ReadParameter(unsigned char Param)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen     = 0x03;
  AirCon.PayLoadFrame.Title        = COMMAND_READ_PARAM;
  AirCon.PayLoadFrame.Parameter[0] = Param;

  AirCon.PayLoad = PayLoad_Start;

  AirCon.LastParam = Param;;

}

/*******************************************************************/
/* Versetzt das Steuergeraet in den Diagmode                       */
/*******************************************************************/
void kl_Command_Set_Diagmode(void)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen = 0x02;
  AirCon.PayLoadFrame.Title    = COMMAND_SET_DIAG;

  AirCon.PayLoad = PayLoad_Start;


}

/*******************************************************************/
/* Verlaesst den Diagmode wieder                                   */
/*******************************************************************/
void kl_Command_Clear_Diagmode(void)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen = 0x02;
  AirCon.PayLoadFrame.Title    = COMMAND_EXIT_DIAG;

  AirCon.PayLoad = PayLoad_Start;


}


/*******************************************************************/
/* Setzt einen Actuator im Steuergeraet. Dazu ist es notwendig     */
/* das sich das Steuergeraet zuvir im actice_diag-mode befindet    */
/* Grundsaetzlichwuerde das Steuergeraet wohl auch mehre Befehle   */
/* in einem Kommando akzeptieren, aber wir nutzen nur einen.       */
/* Erwartet dazu den Parameter der gelesen werden soll             */
/*******************************************************************/
void kl_Command_Set_Actuator(unsigned char Code, unsigned char Param)
{
  /* Nur wenn wir verbunden sind koennen wir Daten anfordern */
  if (AirCon.Status != Connected) return;


  AirCon.PayLoadFrame.Blocklen     = 0x05;
  AirCon.PayLoadFrame.Title        = COMMAND_SET_ACTUATOR;
  AirCon.PayLoadFrame.Parameter[0] = 0x01;  /* 1 Kommando */
  AirCon.PayLoadFrame.Parameter[1] = Code;  /* Aktuator   */
  AirCon.PayLoadFrame.Parameter[2] = Param; /* Daten      */

  AirCon.PayLoad = PayLoad_Start;

}
