#ifndef MAIN
#define WHERE extern
#else
#define WHERE
#endif

/* Addresse unsere Steuergeraetes am Bus */
#define  KL_ADRESS_CLIMA 0x07

/* Befehle die das Steuergeraet kennt */
#define  COMMAND_READ_STATUS  0x40
#define  COMMAND_READ_ERROR   0x45
#define  COMMAND_CLEAR_ERROR  0x46
#define  COMMAND_READ_PARAM   0x47
#define  COMMAND_SET_DIAG     0x49
#define  COMMAND_SET_ACTUATOR 0x42
#define  COMMAND_EXIT_DIAG    0x4a

/* Fehlermeldungen der physikalischen Verbindung */
#define  No_Error       0
#define  BUS_Timeout    1
#define  FrameSize      2
#define  No_Data        3
#define  No_Checksum    4
#define  Wrong_Checksum 5


#define  Parameter_Len 18
struct ISO_Frame {
  unsigned char Blocklen;
  unsigned char Title;
  unsigned char Parameter[Parameter_Len];
  unsigned char Checksum;
  unsigned char Error;
};

/* Fehlermeldung im Datenaustausch mit dem Steuergeraet */
#define Not_connected 0
#define No_Preamble   1
#define Timeout       2
#define Frame_Error   3
#define Connected     4

/* Werte der Zustandsmachine fuer den Paket/Frame austausch */
#define State_Stop    0
#define State_Start   1
#define State_Waiting 2

/* Werte der Zustandsmachine fuer den Austausch der Nutzdaten */
#define PayLoad_Start 1
#define PayLoad_Ready 0
#define PayLoad_Wait  2

#define Preamble_Len 6
struct Clima_Type{
  unsigned char Preamble[Preamble_Len];  /* Preamble des Steuergeraetes */
  struct ISO_Frame     DataFrame;        /* DatenFrame                  */
  struct ISO_Frame     PayLoadFrame;     /* Zweiter DatenFrame          */
  unsigned char Status;                  /* Status der Verbindung       */
  unsigned char State;                   /* Statemachine fuers Daten-PingPong */
  unsigned short FrameCounter;           /* Zaehlt wieviele Frames uebertragen wurden */
  unsigned char PayLoad;                 /* Zeigt an das wir Nutzdaten anfordern */
  unsigned char SerNum[9];               /* Seriennummer des Steuergeraetes */
  unsigned char HW_Version;              /* Hardwareversion */
  unsigned char SW_Version;              /* Softwareversion */
  unsigned char Week[3];                 /* Herstellungswoche */
  unsigned char Year;                    /* Herstellungsjahr */
  unsigned char LastParam;               /* Zuletzt angeforderter Parameter */
  unsigned char Active_Diag;             /* Steuergeraet im Active-Modus? */
};

WHERE struct Clima_Type AirCon;



/* Bekannte Fehlerarten */
#define TREAT_AIR_TEMP1           0x10
#define TREAT_AIR_TEMP2           0x20
#define EXT_TEMP_SENSOR           0x30
#define PASS_COMP_TEMP            0x40
#define AIR_MIX_ACT               0x50
#define AIR_DIST_ACT              0x60
#define INT_FAN                   0x70
#define MIX_ACT_FEEDBACK          0x80
#define DIST_ACT_FEEDBACK         0x90
#define PASS_COMP_SEN_FAN         0xa0
#define COMPRESS_CONTROL          0xb0
#define SUN_SENSOR                0xc0
#define AIR_RECIRC                0xd0
#define IGE_FPLAMP                0x38
#define IGE_RPLAMP                0x48
#define IGE_RFLAMP                0x58
#define IGE_STOPLAMP              0x68
#define IGE_PLATELAMP             0x78
#define IGE_STOPFUSE              0x88
#define ENGINE_OIL                0xa8
#define REAR_GLASS                0xb8
#define EMERGENCY_LIGHT           0xc8
#define MOTRONIC                  0xd8
#define RPM_SIGNAL                0xe8

/* DAs sind die Parameter die einzeln aus dem Steuergeraet */
/* gelesen werden koennen */
#define PARAM_AIR1     0x00
#define PARAM_AIR2     0x01
#define PARAM_EXAIR    0x02
#define PARAM_PASSAIR  0x03
#define PARAM_DISTPOT  0x04
#define PARAM_MIXPOT   0x05
#define PARAM_FAN      0x06
#define PARAM_SPEED    0x07
#define PARAM_SUN      0x08
#define PARAM_READY    0xff


/* Das sind die Actuatoren die wir im Steuergeraet einschalten koennen */
#define ACTUATOR_FAN    0x01
#define ACTUATOR_CIRC   0x02
#define ACTUATOR_DIST   0x03
#define ACTUATOR_MIXER  0x04
#define ACTUATOR_MOTOR  0x05
#define ACTUATOR_LIGHT  0x06
#define ACTUATOR_BUZZER 0x07


void kl_init(void);
int  kl_Frame_exchange(void);
int  kl_klimaconnect(void);
int  kl_CommREADY(void);
void kl_Command_ReadStatus(void);
void kl_Command_ReadError(void);
void kl_Command_ClearError(void);
void kl_Command_ReadParameter(unsigned char Param);
void kl_Command_Set_Diagmode(void);
void kl_Command_Set_Actuator(unsigned char Code, unsigned char Param);
void kl_Command_Clear_Diagmode(void);

void kl_test(void);
