#pragma pack(2)

#include <stdio.h>
#include <string.h>
#include <Pilot.h>
#include <Common.h>
#include <System/SysAll.h>
#include <System/SerialMgr.h>
#include <UI/UIAll.h>
#include <System/SystemMgr.h>

/* Eigene Header */
#include "header.h"       /* Zur Uebergabe der ResourcenIDs */
#include "rs232.h"
#include "AirCon.h"

#include "dragonball.h"

/*Globale Variable *************************************************/
Err RS232_Fehler;         /* Fuer Fehlermeldungen der RS232        */
UInt SerialPointer=0;       /* Zeiger auf die benutzte Schnittstelle */
/*******************************************************************/

/********************************************************************/
/* Serielle Schnittstelle auf die gewuenschten Parameter einstellen */
/********************************************************************/
void SetSerialSettings(UInt SerialPointer, ULong Baud)
{ SerSettingsType SerialSettings;

  SerialSettings.baudRate   = Baud; 
  SerialSettings.ctsTimeout = serDefaultCTSTimeout;
  SerialSettings.flags      = serSettingsFlagStopBits1 | 
                              serSettingsFlagBitsPerChar8;

  /* Kein Handshake */
  SerialSettings.flags &=     ~serSettingsFlagRTSAutoM;
  SerialSettings.flags &=     ~serSettingsFlagCTSAutoM;

  SerSetSettings(SerialPointer, &SerialSettings);
}


/********************************************************************/
/* Sendet einen String und wartet bis zum Ende der Uebertragung     */
/********************************************************************/
Err SendSerialString(UInt SerialPointer, char *data)
{ Err Fehler;

  SerSend(SerialPointer, data, StrLen(data), &Fehler );
  SerClearErr(SerialPointer); 
  SerSendWait(SerialPointer, -1);
  return(Fehler);
};


/********************************************************************/
/* Sendet ein Byte und wartet bis zum Ende der Uebertragung         */
/********************************************************************/
Err SendSerialByte(UInt SerialPointer, unsigned char data)
{ Err Fehler;

  SerSend(SerialPointer, &data, 1, &Fehler );
  SerClearErr(SerialPointer); 
  SerSendWait(SerialPointer, -1);
  return(Fehler);
};



/********************************************************************/
/* Setzt die RTS-Leitung der RS232 im Dragonball                    */
/* Dies fuehrt dazu das die RTS-Leitung am Connector auf +4.8V geht */
/********************************************************************/
void SetSerialRTS(void)
{
  UMISCL |= 0x40; /* Zu Testzwecken fuer Burkhart. */
  PMDATA &= ~0x02;
};


/********************************************************************/
/* Loescht die RTS-Leitung der RS232 im Dragonball                  */
/* Dies fuehrt dazu das die RTS-Leitung am Connector auf -4.8V geht */
/********************************************************************/
void ResetSerialRTS(void)
{
  UMISCL &= ~0x40; /* Auch zu testzwecken */
  PMDATA |= 0x02;
};


/********************************************************************/
/* Setzt die TXD-Leitung der RS232 im Dragonball durch senden       */
/* eines BREAKS. Das setzen des direkten Portsbits klappt leider    */
/* das sich der Palmpilot sofort weghaengt wenn man den TxD-Ausgang */
/* Als Portleitung umprogrammiert                                   */
/* Dies fuehrt dazu das die TXD-Leitung am Connector auf +6V geht   */
/********************************************************************/
void SetSerialTXD(void)
{
  /*   PKDATA |= (Byte) 0x01; */
  /*  UTX |=0x1000; */
  UTXH |=0x10;
};

/********************************************************************/
/* Loescht die TXD-Leitung der RS232 im Dragonball, durch aufheben  */
/* des Break.                                                       */
/* Dies fuehrt dazu das die TXD-Leitung am Connector auf -4.8V geht */
/********************************************************************/
void ResetSerialTXD(void)
{
  /*   PKDATA &= ~(Byte)0x01; */
  /*  UTX &=~0x1000; */
    UTXH &=~0x10; 
};


void InitBitport(void)
{
  PMSEL |= 0x02;
  PMDIR |= 0x02;
}

void ReleaseBitport(void)
{
    PMSEL &= ~0x02;
}


/*******************************************************************/
/* Schliesst die RS232 falls sie benutzt wurde                     */
/*******************************************************************/
void CloseRS232(void)
{

  /* Alte Abschaltzeit wieder einstellen */
   SysSetAutoOffTime (3*60);

  if (SerialPointer==0)
    {
      /* Wir schliessen nur die Schnittstelle wenn sie auch offen ist! */
       return;
    }

  if (RS232_Fehler != sysErrLibNotFound) SerClose(SerialPointer); 

  SerialPointer=0;
  ReleaseBitport();
}


/*******************************************************************/
/* Oeffnet die RS232 falls es ueberhaubt eine gibt und falls sie   */
/* nicht gerade von einem anderen Programm benutzt wird            */
/*******************************************************************/
int OpenRS232(void)
{ 


  if (SerialPointer!=0)
    {
       return(TRUE);
    }

  /*Zeiger fuer RS232 Libary vom System holen */
  RS232_Fehler= SysLibFind("Serial Library", &SerialPointer); 

 /* Pruefen ob es ueberhaubt eine RS232 gibt, ein USB-Palm */
 /* haette oben wahrscheinlich eine FEhlermeldung geliefert*/
 if (RS232_Fehler != sysErrLibNotFound)
 {
   RS232_Fehler= SerOpen(SerialPointer, 0, KappaBaud);  /* Port oeffnen */ 

    /* War die Schnittstelle etwa schon offen? */
   if (RS232_Fehler != serErrAlreadyOpen)
     {
       SetSerialSettings(SerialPointer, KappaBaud );  /* Para einstellen  */ 
     } 
   else 
     {
       FrmAlert(AlertID_RS232_2);       /*Jemand anderes benutzt RS232 */
       SerClose(SerialPointer); 
       return(FALSE);
     };
 } 
 else 
   {
     FrmAlert(AlertID_RS232_1);  /* Port lies sich nicht oeffnen */
     return(FALSE);
   };

 InitBitport();

 /* Da nun ja alles zu laufen scheint, setzen wir mal das AutoOff */
 /* des Palm nach oben damit uns nicht die Kiste andauernd aus geht. */
 SysSetAutoOffTime (20*60);

 return(TRUE);
}



/*******************************************************************/
/* Nur zu Testzwecken                                              */
/*******************************************************************/
void rs232_test(void)
{   
  char      dummy[40];

  strcpy(dummy,"Hallo"); 


   if (RS232_Fehler != sysErrLibNotFound) SendSerialString(SerialPointer,dummy); 

}

/*******************************************************************/
/* Loescht den Buffer der seriellen Schnittstelle                  */
/*******************************************************************/
void  rs232_buffclr(void)
{

  SerReceiveFlush(SerialPointer,1);

}

/*******************************************************************/
/* Liest eine bestimmte Anzahl von Bytes von der RS232             */
/* Parameter:                                                      */
/*                buffer   Datenbuffer der empfangenen Bytes       */
/*                count    Maximale anzahl von Bytes               */
/*                timeout  Maximale Wartezeit in 1/10 Sekunden     */
/*******************************************************************/
int rs232_receive(void *buffer, ULong count, Long timeout)
{ Err   Fehler;
  Long WarteZeit;

 WarteZeit = ( SysTicksPerSecond() * timeout) / 10;
     
 SerReceive(SerialPointer, buffer, count, WarteZeit, &Fehler);

 if (Fehler == 0) return(TRUE);


 return(FALSE);
} 

/*********************************************************************/
/* Sendet ein einzelnes Byte und wartet NICHT bis es abgeschickt ist */
/*********************************************************************/
int rs232_bytesend(unsigned char data)
{ Err   Fehler;

 SerSend(SerialPointer,&data,1, &Fehler);

 if (Fehler == 0) return(TRUE);

 return(FALSE);
}

/********************************************************************/
/* Sendet mehere Bytes und wartet nicht bis sie alle gesendet sind  */
/********************************************************************/
int rs232_buffersend(char *data, int Laenge)
{ Err Fehler;

  SerSend(SerialPointer, data, Laenge, &Fehler );

  if (Fehler == 0) return(TRUE);

 return(FALSE);

};


/*******************************************************************/
/* Wartet darauf das unser Sendepuffer leer wird                   */
/*                timeout  Maximale Wartezeit in 1/10 Sekunden     */
/*******************************************************************/
int rs232_wait(Long timeout)
{
  Long WarteZeit;

  WarteZeit = ( SysTicksPerSecond() * timeout) / 10;

  if (SerSendWait(SerialPointer, WarteZeit) == 0) return(TRUE);

 return(FALSE);


}
